// ****************************************************************
//
//   IBM Confidential
//   OCO Source Materials
//   <<PART NUMBER - 5724-B75>>
//   (C) Copyright IBM Corp. 2001 - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// ****************************************************************

package com.ibm.etools.cobol.importer;


import java.util.*;

/**
 *   The parser instantiates anobject of this class each time it encounters an
 *   elementary item in the COMMAREA field of the CICS transaction.
 */

public class CobolElementaryRecord extends CobolRecord
{
	private static String copyright = "(c) Copyright IBM Corporation 1998, 1999.";
   CobolElementaryRecord(CobolRecord owner, String name, int level)
   {
	  super(owner, name, level);
   }   
void add(CobolRecord record) throws CobolException
{
	// cannot add a record to an elementary item
	throw new CobolException(CobolResource.getString("_EXC_Internal_error"));
}
   String alignment()
   {
	  if (!alignment_)
		 return ((CobolGroupRecord)owner()).alignment();
	  else
	  {
		 if (alignment_)
			return "ALIGN";
		 else
			return " ";
	  }
   }   
   int sign()
   {
	  if (sign_ == NOSIGN)
		 return ((CobolGroupRecord)owner()).sign();
	  else
		 return sign_;
   }   
void trace()
{
	CobolResource.display("Elementary Item: " + name_);
	CobolResource.display("pic string:      " + picString_);
	CobolResource.display("Java Type:       " + javaType_.getName());
	CobolResource.display("alignment:       " + alignment_);
	CobolResource.display("usage:           " + usage_);
	CobolResource.display("sign:            " + sign_);
	CobolResource.display("cobolType:       " + cobolType());
	CobolResource.display("bound:           " + arrayBound_);
	CobolResource.display("filler:          " + filler_);
	CobolResource.display("value:           " + value_);
	CobolResource.display("figConst:        " + figConst_);
}
   int usage()
   {
	  if (usage_ == NOUSAGE)
		 return ((CobolGroupRecord)owner()).usage();
	  else
		 return usage_;
   }   
/**
 * Stores information in the tree that is needed for DRT creation.
 * Creation date: 
 * @param aNonRedefineList Vector - nodes of the tree that don't contain REDEFINE statements
 * @param aNonDependingList Vector - nodes of the tree that don't contain ODO statements
 * @param qualifiedName Vector
 * @param nestedODO boolean - true if this node of the tree is inside an ODO
 */
void walkTree(Vector aNonRedefineList, Vector aNonDependingList, Vector qualifiedName, boolean nestedODO) 
{
	String augmentorStr = cobolType();
	
	/*
	 * If the augmentor is "COMP5,"
	 * change it to "COMP" for the DRT,
	 * since the DRT treats COMP-5 as COMP
	 */
	 //GIM defect 19778: DRT may treat it as COMP, but the CAM model 
	 //needs to set the correct TRUNC attributes for COMP-5s.  Otherwise
	 //COMP and COMP5 cannot be distinguished.
	 //In CAM, if TRUNC(STD) compiler option is in effect,
	 // COMP will have its TRUNC attribute set to
	 //STD but COMP-5 will need to have TRUNC set to BIN.
	 //If TRUNC(BIN) compiler option is in effect, both COMP items are treated
	 //as BINARY with TRUNC attribute set to BIN. Therefore
     //comment out the if stmt below 
     //23920 will fix this up when the EABLIB creates its DRTs that can't correctly
 	 //handle the size and contentSize for COMP5.
//	if(augmentorStr.equals("COMP5")) {
//		augmentorStr = "COMP";
//	}
	
	String alignmentStr = alignment();

	// Sets the fully qualified name
	if (nestedODO)
	{
		qualifiedName.addElement("0");
	}
	qualifiedName.addElement(name_);
	qualifiedName_ = new String[qualifiedName.size()];
	qualifiedName.copyInto(qualifiedName_);

	// Sets the type
	cobolTypeString_ = new String();
	if ( picString_ != " " )
		cobolTypeString_ = picString_;
			
	if ( alignmentStr != " " )
		cobolTypeString_ = cobolTypeString_ + ":" + augmentorStr + ":" + alignmentStr;
	else
		cobolTypeString_ = cobolTypeString_ + ":" + augmentorStr;
			
	// Adds the name of this array to the Cobol Record of the variable that 
	// defines the size of the array
	if (dependedOnItem_.size() > 0)
	{
		for (int i=0; i<aNonDependingList.size(); i++){
			CobolRecord rec = (CobolRecord)aNonDependingList.elementAt(i);
			if (sameItem(rec.qualifiedName_, dependedOnItem_)){
				rec.setVarLengthArrayName(qualifiedName_);
				break;
			}
		}
	}
	else {
		aNonDependingList.addElement(this);
	}
	// Adds this CobolRecord to the CobolRecord that is
	// being redefined by this elementary item
	if ( redefinedItem_ != null )
	{
		for (int i=0; i<aNonRedefineList.size(); i++)
		{
			CobolRecord rec = (CobolRecord)aNonRedefineList.elementAt(i);
			String str = rec.name();
			int num = rec.level();
			if ( num == level_ && str.equalsIgnoreCase(redefinedItem_) )
			{
			   rec.addRedefineType(this);
			   break;
			}
		 }
	}
	else
	{
		aNonRedefineList.addElement(this);
	}
	if (nestedODO)
	{
		qualifiedName.removeElementAt(qualifiedName.size()-1);
	}
	qualifiedName.removeElementAt(qualifiedName.size()-1);
	return;	
}
}
