//****************************************************************
//
//   IBM Confidential
//   OCO Source Materials
//   <<com.ibm.etools.cobol>>
//   (C) Copyright IBM Corp. 2004 - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// ****************************************************************

package com.ibm.etools.cobol.importer;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceImpl;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.internal.Workbench;

import com.ibm.etools.cobol.COBOLClassifier;
import com.ibm.etools.cobol.COBOLComposedType;
import com.ibm.etools.cobol.COBOLElement;
import com.ibm.etools.cobol.plugin.CobolPlugin;
import com.ibm.etools.logging.util.Level;

/**
 * @author Mahendra Akkina (Mahi)
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class CobolSourceResourceWin extends XMIResourceImpl {

	private static HashMap cobolOptions;
	
	private RunCommand runCommand;
	
	private Vector resourcesList;	
	
	/**
	 * 
	 */
	public CobolSourceResourceWin() {
		this(null);
	}

	/**
	 * CobolSourceResourceWin constructor comment.
	 */
	public CobolSourceResourceWin(URI arg) {
		super(arg);	
	}

	public void doLoad(InputStream inputStream, Map options) throws CobolException {
		File tmpFolder=null;
		String uri = getURI().toFileString();
		CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): URI = " + uri);

		/*
		 * Get the default values from the importer preference page
		 */
		cobolOptions = CobolPreferenceStore.getValues();		
		try {
			if(options != null) {
				/* 
				 * Override the default options with those specified by 
				 * the caller.  This ensures all options have a value.
				 */
				Map userOptions = options;
				/* 
				 * Loop thru all the options replacing the defaults.
				 */
				Set npSet = userOptions.keySet();
				for (Iterator i = npSet.iterator(); i.hasNext(); ) {
					String key = (String)i.next();
					Object original = cobolOptions.get(key);
					if (original != null) {
						/*
						 * Make sure the value for the key is valid.
						 */
						if (key.equals(ICobolPreferenceConstants.PRE_COBOL_CODEPAGE)) {
							CobolTypeUtilities.validateCodepage((String)userOptions.get(key));
						} 
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_ENDIAN)) {
							String endian = (String)userOptions.get(key);
							if (endian != null && !endian.equals(ICobolPreferenceConstants.ENDIAN_BIG) && 
								!endian.equalsIgnoreCase(ICobolPreferenceConstants.ENDIAN_LITTLE)) {
								Object[] obj = { endian, key };
								throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj));
							}
						}
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_EXT_DECIMAL_SIGN)) {
							String sign = (String)userOptions.get(key);
							if (sign != null && !sign.equals(ICobolPreferenceConstants.DECIMAL_ASCII) && 
								!sign.equals(ICobolPreferenceConstants.DECIMAL_EBCDIC) &&
								!sign.equals(ICobolPreferenceConstants.DECIMAL_EBCDICC) ) {
								Object[] obj = { sign, key };
								throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj));
							}
							if (sign.equals(ICobolPreferenceConstants.DECIMAL_EBCDICC)) {
                                String codepage_ec = (String)userOptions.get(ICobolPreferenceConstants.PRE_COBOL_HOST_CODEPAGE_EC);
                                if (codepage_ec != null) {
                                    CobolTypeUtilities.validateCodepage(codepage_ec);
                                    cobolOptions.put(ICobolPreferenceConstants.PRE_COBOL_HOST_CODEPAGE_EC, codepage_ec);
                                }
							}
						}
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_FLOATING_POINT_FORMAT)) {
							Integer fpf = (Integer)userOptions.get(key);
							if (fpf.intValue() !=0 && fpf.intValue() != 1) {
								Object[] obj = { fpf, key };
								throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj));
							}
						}
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_NSYMBOL)) {
							String sym = (String)userOptions.get(key);
							if (sym != null && !sym.equals(ICobolPreferenceConstants.NSYMBOL_DBCS) && 
								!sym.equals(ICobolPreferenceConstants.NSYMBOL_NATIONAL) ) {
								Object[] obj = { sym, key };
								throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj));
							}
						}
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_QUOTE)) {
							String quote = (String)userOptions.get(key);
							if (quote != null && !quote.equals(ICobolPreferenceConstants.QUOTE_SINGLE) && 
								!quote.equals(ICobolPreferenceConstants.QUOTE_DOUBLE) ) {
								Object[] obj = { quote, key };
								throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj));
							}
						}
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_REMOTE_ENDIAN)) {
							String endian = (String)userOptions.get(key);
							if (endian != null && !endian.equals(ICobolPreferenceConstants.ENDIAN_BIG) && 
								!endian.equals(ICobolPreferenceConstants.ENDIAN_LITTLE) ) {
								Object[] obj = { endian, key };
								throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj));
							}
						}
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_TRUNC)) {
							String trunc = (String)userOptions.get(key);
							if (trunc != null && !trunc.equals(ICobolPreferenceConstants.TRUNC_BIN) && 
								!trunc.equals(ICobolPreferenceConstants.TRUNC_OPT) &&
								!trunc.equals(ICobolPreferenceConstants.TRUNC_STD) ) {
								Object[] obj = { trunc, key };
								throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj));
							}
						}
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_COMPILE_TIME_LOCALE)) {
							Integer ctl = (Integer)userOptions.get(key);
							COBOLLocaleRelatedConstants clrc = new COBOLLocaleRelatedConstants();			
							if (ctl.intValue() < 0 ||  ctl.intValue()  > (clrc.getLocaleNames()).size() - 1) {
								Object[] obj = { ctl, key };
								throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj));
							}							
						}
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_ASCII_CODEPAGE)) {
							Integer ac = (Integer)userOptions.get(key);
							COBOLLocaleRelatedConstants clrc = new COBOLLocaleRelatedConstants();
							String ctl;
							if (userOptions.containsKey(ICobolPreferenceConstants.PRE_COBOL_COMPILE_TIME_LOCALE))
								ctl = (String) clrc.getLocaleNames().get(((Integer)userOptions.get(ICobolPreferenceConstants.PRE_COBOL_COMPILE_TIME_LOCALE)).intValue());
							else
								ctl = (String) clrc.getLocaleNames().get(((Integer)cobolOptions.get(ICobolPreferenceConstants.PRE_COBOL_COMPILE_TIME_LOCALE)).intValue());							
							if (ac.intValue() < 0 ||  ac.intValue()  > (clrc.getASCIICodePages(ctl)).size() - 1) {
								Object[] obj = { ac, key };
								throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj));
							}														
						}
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_ERROR_MSGS_LANG)) {
							Integer eml = (Integer)userOptions.get(key);
							COBOLLocaleRelatedConstants clrc = new COBOLLocaleRelatedConstants();			
							if (eml.intValue() < 0 ||  eml.intValue()  > (clrc.getErrorMessagesLanguage()).size() - 1) {
								Object[] obj = { eml, key };
								throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj));
							}														
						}
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_CURRENCY)) {
								String c = (String) userOptions.get(key);
								try{									
								CobolTypeUtilities.validateCurrency(c);
								} catch (Exception e) {
									Object[] obj = { c, key };
									throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj)); 
								}
						}
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_COLLSEQ)) {
							String collseq = (String)userOptions.get(key);
							if (collseq != null && !collseq.equals(ICobolPreferenceConstants.COLLSEQ_BIN) && 
								!collseq.equals(ICobolPreferenceConstants.COLLSEQ_EBCDIC) &&
								!collseq.equals(ICobolPreferenceConstants.COLLSEQ_LOCALE) ) {
								Object[] obj = { collseq, key };
								throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj));
							}							
						}
						else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_NCOLLSEQ)) {							
							String ncollseq = (String)userOptions.get(key);
							if (ncollseq != null && !ncollseq.equals(ICobolPreferenceConstants.NCOLLSEQ_BIN) && 
								!ncollseq.equals(ICobolPreferenceConstants.NCOLLSEQ_LOCALE) ) {
								Object[] obj = { ncollseq, key };
								throw new CobolException(CobolResource.getString("_EXC_Invalid_Value_for_Option", obj));
							}							
						}						
						cobolOptions.put(key, userOptions.get(key));
					} else {
						Object[] obj = { key };
						throw new CobolException(CobolResource.getString("_EXC_Invalid_Option", obj));
					}							
				}		 				
			}	    
			String tmpFolderName = (new Long((new java.util.Date()).getTime())).toString();
			tmpFolder = new File(CobolPlugin.getPlugin().getStateLocation().toOSString() + "/" + tmpFolderName);			
			//If the temp folder by this name already exists, try creating another one
			while (tmpFolder.exists()){
				tmpFolderName = (new Long((new java.util.Date()).getTime())).toString();	
				tmpFolder = new File(CobolPlugin.getPlugin().getStateLocation().toOSString() + "/" + tmpFolderName);    	
			}
			if (tmpFolder.mkdir())
				CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): Created Directory " + tmpFolder.getName() + " Successfully");
			else {
				CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): Creation of Directory " + tmpFolder.getName() + " Failed", new CobolException(CobolResource.getString("_Error_Creating_Folder")));		
				throw new CobolException(CobolResource.getString("_Error_Creating_Folder"));
			}
	
	
			IPath absPath = new Path (uri);
			if(isRFile(absPath)) {
				CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): Selected file is a remote file");
				runCommand = new RunCommand(uri, tmpFolder.getPath());				
				runCommand.createXMI(cobolOptions);
				String xmlFilename = tmpFolder.getPath()+"/syntaxerrors.xml";				
				String xmlCombinedFilename = tmpFolder.getPath()+"/syntaxerrors_complete.xml";				
				ParseSyntaxErrorsXMLOutput parseErrors = new ParseSyntaxErrorsXMLOutput(xmlFilename, xmlCombinedFilename);
				if ((new File(xmlCombinedFilename)).exists()) {
					parseErrors.parseXMLOutputFile(null);
					if (!parseErrors.hasErrors()){				
						String fullPath = tmpFolder.getPath() +  "\\cobol.xmi";
						CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): XMI File location = " + fullPath);			
						//Check if the file exists     
						if ((new File(fullPath)).exists())
							CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): XMI Exists");
						else {
							CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): XMI file not found.");		
							throw new CobolException(CobolResource.getString("_XMI_File_Not_Found"));
						}	    
						loadXMI(fullPath);					
                        System.out.println("tmpFolder = " + tmpFolder.getPath());
						CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): importerResource = " + this.toString());			    	
					} else {
						  CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): Syntax errors found. Check in the task list.");
						  CobolException ce = new CobolException(CobolResource.getString("_Syntax_Errors_Found_For_Remote_File"));
						  ce.setErrorMessagesVector(parseErrors.getErrorMessagesVector());							  
					  		throw ce;
					}
				} else {
					  CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): "+ xmlCombinedFilename+" file does not exist, please try again.");
					  throw new CobolException(CobolResource.getString("_Syntax_Error_File_Not_Found"));	
				}
			
			} else {	
				String resourceFileString = null;
				
				IFile typesFile = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(absPath);

				if (typesFile == null) {
					resourceFileString = absPath.toString();
					CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): File is not an IFile = " + absPath.toString());
				}else {
					resourceFileString = typesFile.getLocation().toString();
					CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): Selected IFile = " + typesFile);				
					CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): Selected resource file location = " + resourceFileString);
				}  
				if ((resourceFileString.substring(resourceFileString.lastIndexOf('/')+1, resourceFileString.length())).length() > 64) {    	
					CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** RunCommand.createXMI(Hashmap): length of the file > 60 ="); 								    	
					throw new CobolException(CobolResource.getString("_File_Name_Too_Long"));			
				}
				runCommand = new RunCommand(resourceFileString, tmpFolder.getPath());				
				runCommand.createXMI(cobolOptions);
				String xmlFilename = tmpFolder.getPath()+"/syntaxerrors.xml";				
				String xmlCombinedFilename = tmpFolder.getPath()+"/syntaxerrors_complete.xml";				
				ParseSyntaxErrorsXMLOutput parseErrors = new ParseSyntaxErrorsXMLOutput(xmlFilename, xmlCombinedFilename);				
				if ((new File(xmlCombinedFilename)).exists()) {
					parseErrors.parseXMLOutputFile(typesFile);
					resourcesList = parseErrors.getResourceVector();					
					if (!parseErrors.hasErrors()){			
						String fullPath = null;
						if (typesFile == null)
							fullPath = tmpFolder.getPath() + "\\cobol.xmi"; 
						else {
							int dotIndex = typesFile.getName().lastIndexOf(".");	
							fullPath = tmpFolder.getPath() + "\\cobol.xmi";							
						}

						CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): XMI File location = " + fullPath);			
						//Check if the file exists
						if ((new File(fullPath)).exists())
							CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): XMI Exists");
						else {
							CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): XMI file not found.");		
							throw new CobolException(CobolResource.getString("_XMI_File_Not_Found"));
						}	
						
						loadXMI(fullPath);			
						
						if (typesFile != null)			
							typesFile.getParent().refreshLocal(IResource.DEPTH_INFINITE, null);			
						CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): importerResource = " + this.toString());			    	
					} else {						  
						  CobolException ce;
						  if (typesFile != null) {
		    				  typesFile.getParent().refreshLocal(IResource.DEPTH_INFINITE, null);
		    				  /* Open the Problems task list view if the workbench is running and it is not currently open */
		    				  if (Workbench.getInstance() != null  && Workbench.getInstance().isRunning()) {
		    				  	CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): Workbench is running");
		    				  	openProblemsView();
		    				  } else {
		    				  	CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): Workbench is not running. Importer is running in Batch mode");
		    				  }
							  CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): Syntax errors found. Check in the task list.");
							  ce = new CobolException(CobolResource.getString("_Syntax_Errors_Found_For_IFile"));
							  ce.setMessageNote(getMessageNote());
							  ce.setErrorMessagesVector(parseErrors.getErrorMessagesVector());
						  		throw ce;
						  } else {
							CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): Syntax errors found in a non IFile. Check in the syntaxerrors.xml file.");	    	  
							  ce = new CobolException(CobolResource.getString("_Syntax_Errors_Found_For_Non_IFile"));
							  ce.setErrorMessagesVector(parseErrors.getErrorMessagesVector());							  
						  		throw ce;						  	
						  }
					}
				} else {
					  if (typesFile != null)
					  	typesFile.getParent().refreshLocal(IResource.DEPTH_INFINITE, null);
					  CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): "+ xmlCombinedFilename+" file does not exist, please try again.");
					  throw new CobolException(CobolResource.getString("_Syntax_Error_File_Not_Found"));	
				}
			}
			List ele = getCobolTopElements();
			if (ele.isEmpty()) {
				  CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): Syntax errors found. Check in the task list.");
				  CobolException ce = new CobolException(CobolResource.getString("_Syntax_Errors_Found_For_IFile"));
				  ce.setMessageNote(getNoElementsFoundMessageNote());
				  throw ce;
			}
		} catch(CobolException e) {			
		   throw e;		   
		} catch(Exception e) {			
			CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolSourceResourceWin.load(ResourceSet, String, InputStream , Object): This Exception should not occur: " + e.getMessage(), e);		   
		} finally{
			deleteTempFolder(tmpFolder);
		}
		
	}

	public Vector getResourcesList() {
		return resourcesList;
	}
	
	/**
	 * Open the Problems view if it is not currently open
	 */
	private void openProblemsView() {
		Display.getDefault().asyncExec(
				new Runnable() {
					public void run() {		
					    IWorkbenchWindow[] windows = CobolPlugin.getDefault().getWorkbench().getWorkbenchWindows();
						if (windows !=  null) {
							for (int i = 0; i < windows.length; i ++) {	
								IWorkbenchPage page = windows[i].getActivePage();
								IPageLayout ipl = null;
								if (page != null) {
									IViewPart problemsView = page
											.findView(IPageLayout.ID_PROBLEM_VIEW);
									try {									
										if (problemsView == null) {					
												page.showView(IPageLayout.ID_PROBLEM_VIEW);
												IWorkbenchPart activePart = page.getActivePart();
												page.activate(activePart);
												break;						
										} else {
											page.showView(IPageLayout.ID_PROBLEM_VIEW,null,IWorkbenchPage.VIEW_VISIBLE);
											break;
										}
									} catch (PartInitException e) {
										CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolSourceResourceWin.openProblemsView(): Unable to open the Problems view.", e);						
									}									
								}
							}
						}	
					}
		});								
	}

	/**
	 * @return
	 */
	private String getNoElementsFoundMessageNote() {
		String fileName = getURI().lastSegment();
		Object[] obj = {fileName};
		String errMsg = CobolResource.getString("_Syntax_Errors_Found_For_IFile_Reason_Label") + 
						"\n" + CobolResource.getString("_Syntax_Errors_Found_For_IFile_NoDataReason1", obj) + "\n";	
		errMsg =  errMsg + CobolResource.getString("_Syntax_Errors_Found_For_IFile_NoDataReason2", obj);
		errMsg = errMsg + CobolResource.getString("_Syntax_Errors_Found_For_IFile_MoreInfo");		
		return errMsg;
	}
	
	/**
	 * @return
	 */
	private String getMessageNote() {
		String fileName = getURI().lastSegment();
		String ext = "." + getURI().fileExtension();		
		Object[] obj = {fileName};
		String errMsg = CobolResource.getString("_Syntax_Errors_Found_For_IFile_Reason_Label") + 
						"\n" + CobolResource.getString("_Syntax_Errors_Found_For_IFile_Reason1") + "\n";	
		HashMap extensionSupportMap = runCommand.getExtensionSupportMap();
		if (extensionSupportMap.get(ext).equals(ICobolPreferenceConstants.DEFAULT_ES)) //Full Program
			errMsg =  errMsg + CobolResource.getString("_Syntax_Errors_Found_For_IFile_Reason2a", obj);
		else
			errMsg =  errMsg + CobolResource.getString("_Syntax_Errors_Found_For_IFile_Reason2b", obj);
		
		errMsg = errMsg + CobolResource.getString("_Syntax_Errors_Found_For_IFile_MoreInfo");		
		return errMsg;
	}

	/**
	 *  Delete the folder
	 */
	private void deleteTempFolder(File tmpFolder) {
		boolean deleteStatus = false;
		//System.out.println("Level = " + CobolPlugin.getMsgLogger().getLevel());
		CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.deleteTempFolder(File): Logging Level = " + CobolPlugin.getMsgLogger().getLevel());
		if (CobolPlugin.getMsgLogger().getLevel() <= Level.FINE)
			return;   	
		if (tmpFolder.exists()){
			File[] tempFiles = tmpFolder.listFiles();
			for(int i = 0; i < tempFiles.length; ++i) {
				tempFiles[i].delete();
			}
			deleteStatus = tmpFolder.delete();
			if (deleteStatus)
				CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** CobolSourceResourceWin.deleteTempFolder(File): Temporary folder successfully deleted.");	    	  
			else
				CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolSourceResourceWin.deleteTempFolder(File): Unable to delete the Temporary folder.");
		} else {
				CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** CobolSourceResourceWin.deleteTempFolder(File): Temporary folder does not exist.");
		}		
	}

	public boolean isRFile(IPath p) {
		if (p.isUNC() && p.segment(0).equalsIgnoreCase("ffssash")) {
			return true;
		} else
			return false;
	}
	
	private void loadXMI(String fullPath) throws IOException {
		// Instantiate the appropriate importer
		Resource.Factory rf = Resource.Factory.Registry.INSTANCE.getFactory(URI.createFileURI(fullPath));
		XMLResource importerResource = (XMLResource) rf.createResource(URI.createFileURI(fullPath));
		importerResource.load(null);
		getEObjectToIDMap().putAll(importerResource.getEObjectToIDMap());
		getContents().addAll(importerResource.getContents());		
	}

	/*****************************************************************************************************
	 * Get a list of 01 level cobol elements from a CAM based emf resource
	 *****************************************************************************************************/
	public List getCobolTopElements() {
		List eleList = new ArrayList();

		if (this == null) {
			return eleList;
		}

		Iterator typeIterator = getContents().iterator();

		while (typeIterator.hasNext()) {
			Object possibleType = typeIterator.next();

			// Gather only the Level 01's, which will be identified
			// by a COBOLElement at the top of the extent
			if (possibleType instanceof COBOLElement) {
				COBOLClassifier childClassifier = ((COBOLElement) possibleType).getSharedType();

				//We are interested only in composed types
				if (childClassifier instanceof COBOLComposedType) {
					eleList.add(possibleType);
				}
			}
		} 

		return eleList;
	}
	
}
