// ****************************************************************
//
//   IBM Confidential
//   OCO Source Materials
//   <<PART NUMBER - 5724-B75>>
//   (C) Copyright IBM Corp. 2001, 2003 - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// ****************************************************************
package com.ibm.etools.cobol.importer;

import com.ibm.ivj.eab.record.cobol.CobolType;
import com.ibm.record.RecordException;

/**
 * @author mazo
 *
 * This class is designed to cover the deficiency in EABLIB that can't correctly
 * handle COMP5 types.  For that reason, this class will wrap the EABLIB's class
 * preserve COMP5 for the TRUNC attribute and re-map COMP5 as COMP for EABLIB. 
 * This is a fix for CMVC PE 23920 (contentSize and Size incorrect in the 
 * TDSimpleInstance for COMP-5 after applying fix for defect 19778 .)
 */
public class CobolTypeGov extends CobolType {

    private String augString;
	/**
	 * Constructor for CobolTypeGov.
	 */
	public CobolTypeGov() {
		super();
		augString="";
	}

	/**
	 * Constructor for CobolTypeGov.
	 * @param cobType
	 * @throws RecordException
	 */
	public CobolTypeGov(String cobType) throws RecordException {
		super(cobType);
		augString="";
		String aug = getAug();
		int i = aug.indexOf("COMP5");
        //Keep COMP5 info in this class, but remap it for eablib		
		if (i != -1) {
			augString = aug;
			aug = aug.substring(0,i) + "COMP"+ aug.substring(i+"COMP5".length(), aug.length());
			super.setAug(aug);
		}
	}
	
	/**
	 * Getter for the governing augmentation.
	 */
	public final String getAugGov() {
		return augString;
	}

}
